package org.apache.juli.logging.ch.qos.logback.classic.model.processor;

import org.apache.juli.logging.ch.qos.logback.classic.Level;
import org.apache.juli.logging.ch.qos.logback.classic.Logger;
import org.apache.juli.logging.ch.qos.logback.classic.LoggerContext;
import org.apache.juli.logging.ch.qos.logback.classic.model.RootLoggerModel;
import org.apache.juli.logging.ch.qos.logback.core.Context;
import org.apache.juli.logging.ch.qos.logback.core.joran.spi.InterpretationContext;
import org.apache.juli.logging.ch.qos.logback.core.model.Model;
import org.apache.juli.logging.ch.qos.logback.core.model.processor.ModelHandlerBase;
import org.apache.juli.logging.ch.qos.logback.core.model.processor.ModelHandlerException;
import org.apache.juli.logging.ch.qos.logback.core.util.OptionHelper;

public class RootLoggerModelHandler extends ModelHandlerBase {

	Logger root;
	boolean inError = false;

	public RootLoggerModelHandler(Context context) {
		super(context);
	}

	static public ModelHandlerBase makeInstance(Context context, InterpretationContext ic) {
		return new RootLoggerModelHandler(context);
	}	
	
	protected Class<RootLoggerModel> getSupportedModelClass() {
		return RootLoggerModel.class;
	}

	@Override
	public void handle(InterpretationContext interpretationContext, Model model) throws ModelHandlerException {
		inError = false;

		RootLoggerModel rootLoggerModel = (RootLoggerModel) model;

		LoggerContext loggerContext = (LoggerContext) this.context;
		root = loggerContext.getLogger(Logger.ROOT_LOGGER_NAME);

		String levelStr = interpretationContext.subst(rootLoggerModel.getLevel());
		if (!OptionHelper.isNullOrEmpty(levelStr)) {
			Level level = Level.toLevel(levelStr);
			addInfo("Setting level of ROOT logger to " + level);
			root.setLevel(level);
		}
		
		interpretationContext.pushObject(root);
	}

	@Override
	public void postHandle(InterpretationContext ic, Model model) {
		if (inError) {
			return;
		}
		Object o = ic.peekObject();
		if (o != root) {
			addWarn("The object ["+o+"] on the top the of the stack is not the root logger");
		} else {
			ic.popObject();
		}
	}

}
